import { Collection, Entity, OneToMany, PrimaryKey, Property } from '@mikro-orm/core'
import { ZoneEffect } from './zoneEffect'
import { ZoneEventTile } from './zoneEventTile'
import { ZoneEventTileTeleport } from './zoneEventTileTeleport'
import { ZoneObject } from './zoneObject'
import { Character } from './character'
import { Chat } from './chat'

@Entity()
export class Zone {
  @PrimaryKey()
  id!: number

  @Property()
  name!: string

  @Property()
  width = 10

  @Property()
  height = 10

  @Property({ type: 'json', nullable: true })
  tiles?: any

  @Property()
  pvp = false

  @OneToMany(() => ZoneEffect, (effect) => effect.zone)
  zoneEffects = new Collection<ZoneEffect>(this)

  @OneToMany(() => ZoneEventTile, (tile) => tile.zone)
  zoneEventTiles = new Collection<ZoneEventTile>(this)

  @OneToMany(() => ZoneEventTileTeleport, (teleport) => teleport.toZone)
  zoneEventTileTeleports = new Collection<ZoneEventTileTeleport>(this)

  @OneToMany(() => ZoneObject, (object) => object.zone)
  zoneObjects = new Collection<ZoneObject>(this)

  @OneToMany(() => Character, (character) => character.zone)
  characters = new Collection<Character>(this)

  @OneToMany(() => Chat, (chat) => chat.zone)
  chats = new Collection<Chat>(this)

  @Property()
  createdAt = new Date()

  @Property()
  updatedAt = new Date()
}
